@@ -11,6 +11,8 @@ import tempfile
 import textwrap
 import types
 import unittest
+from io import StringIO
+from unittest import mock, skipIf
 
 # setuptools imports `imp`, which triggers a DeprecationWarning starting with
 # Python 3.4 in the middle of my pristine test suite.  But if I do the import
@@ -18,72 +20,23 @@ import unittest
 # no warning.
 import setuptools  # noqa
 
-try:
-    from unittest import mock
-except ImportError:
-    import mock
-
 import objgraph
 
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from io import StringIO
-
-
-try:
-    from unittest import skipIf
-except ImportError:
-    def skipIf(condition, reason):
-        def wrapper(fn):
-            if condition:
-                def empty_test(case):
-                    pass
-                empty_test.__doc__ = '%s skipped because %s' % (
-                    fn.__name__, reason)
-                return empty_test
-            return fn
-        return wrapper
-
 
 def format(text, **kwargs):
     template = string.Template(text)
     return template.substitute(kwargs)
 
 
-class CompatibilityMixin(object):
-
-    # Python 2.7 .. 3.1 has assertRegexpMatches but not assertRegex
-    # Python <= 2.6 has neither
-    # Python >= 3.2 has both and emits deprecation warnings if you use
-    # assertRegexpMatches.
-
-    if not hasattr(unittest.TestCase, 'assertRegex'):
-        if hasattr(unittest.TestCase, 'assertRegexpMatches'):
-            # This is needed for Python 3.1: let's reuse the existing
-            # function because our replacement doesn't work on Python 3
-            assertRegex = unittest.TestCase.assertRegexpMatches
-        else:
-            def assertRegex(self, text, expected_regexp, msg=None):
-                if isinstance(expected_regexp, basestring):  # noqa
-                    expected_regexp = re.compile(expected_regexp)
-                if not expected_regexp.search(text):
-                    msg = msg or "Regexp didn't match"
-                    msg = '%s: %r not found in %r' % (msg,
-                                                      expected_regexp.pattern,
-                                                      text)
-                    raise self.failureException(msg)
-
-
 class GarbageCollectedMixin(object):
     """A mixin for test cases that garbage collects before running."""
 
     def setUp(self):
-        super(GarbageCollectedMixin, self).setUp()
+        super().setUp()
         gc.collect()
 
     def tearDown(self):
-        super(GarbageCollectedMixin, self).tearDown()
+        super().tearDown()
         gc.enable()
 
 
@@ -378,14 +331,13 @@ def doctest_get_new_ids_prints():
 
         >>> _ = objgraph.get_new_ids(limit=0)
         >>> _ = objgraph.get_new_ids(limit=0)
-        >>> a = [0, 1, 2]  # noqa
-        >>> b = [3, 4, 5]  # noqa
+        >>> a = [[] for n in range(10)]  # noqa
         >>> _ = objgraph.get_new_ids(limit=1)
         ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
         ========================================================
         Type      Old_ids  Current_ids      New_ids Count_Deltas
         ========================================================
-        list          ...          ...          ...           +2
+        list          ...          ...          ...          +11
         ========================================================
 
     """
@@ -422,7 +374,6 @@ class AtAddrsTest(unittest.TestCase):
 
 
 class StringRepresentationTest(GarbageCollectedMixin,
-                               CompatibilityMixin,
                                unittest.TestCase):
     """Tests for the string representation of objects and edges."""
 
@@ -504,6 +455,17 @@ class StringRepresentationTest(GarbageCollectedMixin,
 
         self.assertEqual('a_method', objgraph._short_repr(MyClass.a_method))
 
+    def test_short_repr_frame(self):
+        frame = sys._getframe()
+        # we're calling _short_repr() 6 lines down from here
+        lineno = frame.f_lineno + 6
+        # Python >= 3.9 uses absolute filenames
+        expected = {
+            'tests.py:%d' % lineno,
+            '%s:%d' % (os.path.abspath('tests.py'), lineno),
+        }
+        self.assertIn(objgraph._short_repr(frame), expected)
+
     def test_gradient_empty(self):
         self.assertEqual((0.1, 0.2, 0.3),
                          objgraph._gradient((0.1, 0.2, 0.3),
@@ -514,6 +476,11 @@ class StringRepresentationTest(GarbageCollectedMixin,
         self.assertEqual(' [label="f_locals",weight=10]',
                          objgraph._edge_label(frame, frame.f_locals))
 
+    def test_edge_label_frame_globals(self):
+        frame = sys._getframe()
+        self.assertEqual(' [label="f_globals",weight=10]',
+                         objgraph._edge_label(frame, frame.f_globals))
+
     @skipIf(sys.version_info[0] > 2, "Python 3 has no unbound methods")
     def test_edge_label_unbound_method(self):
         class MyClass(object):
